MODULE Starter 
  !Symbolic names for kind types of 4-, 2-, and 1-byte integers: 
  INTEGER, PARAMETER :: I4B = SELECTED_INT_KIND(9) 
  INTEGER, PARAMETER :: I2B = SELECTED_INT_KIND(4) 
  INTEGER, PARAMETER :: I1B = SELECTED_INT_KIND(2) 
  !Symbolic names for kind types of single- and double-precision reals: 
  INTEGER, PARAMETER :: SP = KIND(1.0) 
  INTEGER, PARAMETER :: DP = KIND(1.0D0) 
  !Symbolic names for kind types of single- and double-precision complex:
  INTEGER, PARAMETER :: SPC = KIND((1.0,1.0)) 
  INTEGER, PARAMETER :: DPC = KIND((1.0D0,1.0D0)) 
  !Symbolic name for kind type of default logical: 
  INTEGER, PARAMETER :: LGT = KIND(.true.) 
  !Frequently used mathematical constants (with precision to spare): 
  REAL(SP), PARAMETER :: PI=3.141592653589793238462643383279502884197_sp 
  REAL(SP), PARAMETER :: PIO2=1.57079632679489661923132169163975144209858_sp 
  REAL(SP), PARAMETER :: TWOPI=6.283185307179586476925286766559005768394_sp 
  REAL(SP), PARAMETER :: SQRT2=1.41421356237309504880168872420969807856967_sp 
  REAL(SP), PARAMETER :: EULER=0.5772156649015328606065120900824024310422_sp 
  REAL(DP), PARAMETER :: PI_D=3.141592653589793238462643383279502884197_dp 
  REAL(DP), PARAMETER :: PIO2_D=1.57079632679489661923132169163975144209858_dp 
  REAL(DP), PARAMETER :: TWOPI_D=6.283185307179586476925286766559005768394_dp 
END MODULE Starter

MODULE Array_input

  USE starter, ONLY : wp=>dp
  IMPLICIT NONE
  Public

  INTERFACE input_from_file
     MODULE PROCEDURE input_from_file_array_r1
     MODULE PROCEDURE input_from_file_array_r2
     MODULE PROCEDURE input_from_file_array_r3
  END INTERFACE

CONTAINS

  subroutine input_from_file_array_r1(a,filename,error)

    INTEGER :: i,n
    INTEGER, INTENT(out) :: error
    REAL(wp) , DIMENSION(:) :: a

    CHARACTER(LEN=*) :: filename
    
    OPEN(UNIT=9,file=filename)
    n = size(a)
    read(9,*)i
    IF(i==n)THEN
        error=0
        READ(9,*)a
    else
        PRINT*,'Error inputting from file : ',filename
        ERROR = 1
        CLOSE(UNIT=9)
        RETURN
    ENDIF
    CLOSE(UNIT=9)

  END SUBROUTINE input_from_file_array_r1
  
  subroutine input_from_file_array_r2(a,filename,error)

    INTEGER :: i,j,n,m
    INTEGER, INTENT(out) :: error
    REAL(wp) , DIMENSION(:,:) :: a

    CHARACTER(LEN=*) :: filename
    
    OPEN(UNIT=9,file=filename)
    n = size(a,1)
    m = size(a,2)
    read(9,*)i,j
    IF(i==n.AND.j==m)THEN
        error=0
        DO J=1,M
            READ(9,*)a(:,j)
        END DO
    else
        PRINT*,'Error inputting from file : ',filename
        ERROR = 1
        RETURN
        CLOSE(UNIT=9)
    ENDIF
    CLOSE(UNIT=9)
    
  END SUBROUTINE input_from_file_array_r2
  
  subroutine input_from_file_array_r3(a,filename,error)

    INTEGER :: i,j,k,n,m,p
    INTEGER, INTENT(out) :: error
    REAL(wp) , DIMENSION(:,:,:) :: a

    CHARACTER(LEN=*) :: filename
    
    OPEN(UNIT=9,file=filename)
    n = size(a,1)
    m = size(a,2)
    p = size(a,3)
    read(9,*)i,j,k
    IF(i==n.AND.j==m.AND.k==p)THEN
        error=0
        DO k=1,p
            DO j=1,m
               READ(9,*)a(:,j,k)
            END DO
        END DO
    else
        PRINT*,'Error inputting from file : ',filename
        ERROR = 1
        close(unit=9)
        RETURN
    ENDIF
    close(unit=9)

  END SUBROUTINE input_from_file_array_r3
  
END MODULE Array_input

MODULE Array_operations

  USE starter, ONLY : wp=>dp
  IMPLICIT NONE
  Public

  INTERFACE l2norm
     MODULE PROCEDURE l2norm_r1
     MODULE PROCEDURE l2norm_r2
     MODULE PROCEDURE l2norm_r3
  END INTERFACE

CONTAINS

  
  FUNCTION l2norm_r1(a)

    INTEGER :: i,n
    REAL(wp) , DIMENSION(:) :: a
    REAL(wp) :: l2norm_r1,work
    
    n = size(a)
    work = 0._wp
    DO i=1,n
        work = work + a(i)
    END DO
    l2norm_r1 = SQRT(work)

  END FUNCTION  l2norm_r1

  FUNCTION l2norm_r2(a)

    INTEGER :: i,J,n,m
    REAL(wp) , DIMENSION(:,:) :: a
    REAL(wp) :: l2norm_r2,work
    
    n = size(a,1)
    m = size(a,2)
    work = 0._wp
    DO i=1,n
       DO j=1,m
          work = work + a(i,j)
       END DO
    END DO
    l2norm_r2 = SQRT(work)

  END FUNCTION  l2norm_r2

  FUNCTION l2norm_r3(a)

    INTEGER :: i,j,k,n,m,p
    REAL(wp) , DIMENSION(:,:,:) :: a
    REAL(wp) :: l2norm_r3,work
    
    n = size(a,1)
    m = size(a,2)
    p = size(a,3)
    work = 0._wp
    DO i=1,n
       DO j=1,m
          DO k=1,p
             work = work + a(i,j,k)
          END DO
       END DO
    END DO
    l2norm_r3 = SQRT(work)

  END FUNCTION  l2norm_r3

 
END MODULE Array_operations

PROGRAM TEST_ARRAYS
  
  USE starter, ONLY : wp=>dp
  USE array_input
  USE Array_operations
  IMPLICIT NONE
  
  INTEGER :: n,m,p,error
  REAL(wp), ALLOCATABLE :: a(:)
  REAL(wp), ALLOCATABLE :: b(:,:)
  REAL(wp), ALLOCATABLE :: c(:,:,:)
  REAL(wp) :: x,y,z
  CHARACTER (LEN=50) :: format_label

  PRINT*,'Input size of array dimensions n, m, and p'
  READ(5,*)n,m,p
  
  ALLOCATE(A(n),B(n,m),C(n,m,p))
  
 CALL Input_from_file(a,'one_dim_array.dat',error)
 IF(ERROR/=0)STOP
 CALL Input_from_file(b,'two_dim_array.dat',error)
 IF(ERROR/=0)STOP
 CALL Input_from_file(c,'three_dim_array.dat',error)
 IF(ERROR/=0)STOP

  x = l2norm(a)
  y = l2norm(b)
  z = l2norm(c)
  
  format_label ='(3(a,f10.8,1x,/))'
  WRITE(6,format_label)'norm(a) :',x,'norm(b) :',y,'norm(c) :',z

END PROGRAM TEST_ARRAYS
